@@ -2,7 +2,7 @@
 from time import sleep
 from dogtail.rawinput import typeText, pressKey, keyCombo, absoluteMotion, click
 from behave import step
-from qecore import get_application, get_application_root
+from qecore.utility import get_application, get_application_root
 from qecore.get_node import GetNode, get_center
 
 
