@@ -7,23 +7,21 @@ from subprocess import Popen
 from dogtail.tree import root
 from dogtail.utils import config
 from dogtail.rawinput import keyCombo
-from qecore import run
+from qecore.utility import run
 from qecore.logger import QELogger
 from qecore.application import Application
+from qecore.flatpak import Flatpak
 
 log = QELogger()
 
 class TestSandbox:
     def __init__(self, component, logging=False):
         """
-        Initiate TestSandbox instance.
+        :type component: str
+        :param component: Name of the component that is being tested.
 
-        @param component : str
-            Testing component name.
-        @param logging : bool
-            Turn on or off logging of this submodule.
-            Default option is False.
-            Mostly used if submodule encounters a corner case that was not caught in implementation.
+        :type logging: bool
+        :param logging: Turn on or off logging of this submodule.
         """
 
         self.logging = logging
@@ -70,15 +68,18 @@ class TestSandbox:
 
         self.shell = root.application("gnome-shell")
 
+        self.preserve_old_api()
+
 
     def before_scenario(self, context, scenario):
         """
         Actions that are to be executed before every scenario.
 
-        @param context : <context_object>
-            Pass this object from environment file.
-        @param scenario : <behave_object>
-            Pass this object from environment file.
+        :type context: <behave.runner.Context>
+        :param context: Pass this object from environment file.
+
+        :type scenario: <Scenario>
+        :param scenario: Pass this object from environment file.
         """
 
         if self.logging:
@@ -109,10 +110,11 @@ class TestSandbox:
         """
         Actions that are to be executed after every scenario.
 
-        @param context : <context_object>
-            Pass this object from environment file.
-        @param scenario : <behave_object>
-            Pass this object from environment file.
+        :type context: <behave.runner.Context>
+        :param context: Pass this object from environment file.
+
+        :type scenario: <Scenario>
+        :param scenario: Pass this object from environment file.
         """
 
         if self.logging:
@@ -156,7 +158,11 @@ class TestSandbox:
 
     def start_recording(self):
         """
-        Start recording video.
+        Start recording the video.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
         """
 
         if self.logging:
@@ -183,7 +189,11 @@ class TestSandbox:
 
     def stop_recording(self):
         """
-        Stop recording video.
+        Stop recording the video.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`after_scenario`.
         """
 
         if self.logging:
@@ -195,31 +205,72 @@ class TestSandbox:
         self.record_video_pid = None
 
 
-    def get_app(self, name, a11yAppName=None, desktopFileExists=True,
-                desktopFileName=None, appProcessName=None):
+    def get_app(self,
+                name,
+                a11yAppName=None,
+                desktopFileExists=True,
+                desktopFileName=None,
+                appProcessName=None):
         """
-        Return application to be used in test.
-
-        @param name : str
-            Name of application.
-        @param a11yAppName : str
-            Name of application in a11y tree.
-        @param desktopFileExists : bool
-            Does desktop file of application exist.
-        @param desktopFileName : str
-            Name of desktop file of application.
-        @param appProcessName : str
-            Name of process after application starts.
+        This function is a wrapper over :func:`get_application` to preserve the old api name.
+        This is defined as an extra function because even the method parameters were renamed to
+        comply with the snake_case naming style.
         """
 
         if self.logging:
-            log.info("".join((
-                f"get_app(self, name={name}, a11yAppName={a11yAppName}, ",
-                f"desktopFileExists={desktopFileExists}, desktopFileName={desktopFileName})"
+            log.info(" ".join((
+                f"get_app(self, name={name}, a11yAppName={a11yAppName},",
+                f"desktopFileExists={desktopFileExists}, desktopFileName={desktopFileName},",
+                f"appProcessName={appProcessName})"
             )))
 
-        new_application = Application(name, a11yAppName, desktopFileExists,
-                                      desktopFileName, appProcessName,
+        return self.get_application(name,
+                                    a11y_app_name=a11yAppName,
+                                    desktop_file_exists=desktopFileExists,
+                                    desktop_file_name=desktopFileName,
+                                    app_process_name=appProcessName)
+
+
+    def get_application(self,
+                        name,
+                        a11y_app_name=None,
+                        desktop_file_exists=True,
+                        desktop_file_name=None,
+                        app_process_name=None):
+        """
+        Return application to be used in test.
+
+        :type name: str
+        :param name: Name of the package that provides the application.
+
+        :type a11y_app_name: str
+        :param a11y_app_name: Application's name as it appears in the a11y tree.
+
+        :type desktop_file_exists: bool
+        :param desktop_file_exists: Does desktop file of the application exist?
+
+        :type desktop_file_name: str
+        :param desktop_file_name: Application's desktop file name.
+
+        :type app_process_name: str
+        :param app_process_name: Application's name as it appears in a running process.
+
+        :return: Application class instance
+        :rtype: <qecore.application.Application>
+
+        This function is wrapped by :func:`get_app`.
+        """
+
+        if self.logging:
+            log.info(" ".join((
+                f"get_application(self, name={name}, a11y_app_name={a11y_app_name},",
+                f"desktop_file_exists={desktop_file_exists},",
+                f"desktop_file_name={desktop_file_name},",
+                f"app_process_name={app_process_name})"
+            )))
+
+        new_application = Application(name, a11y_app_name, desktop_file_exists,
+                                      desktop_file_name, app_process_name,
                                       self.shell, self.session_type,
                                       self.session_desktop, self.logging)
 
@@ -230,9 +281,34 @@ class TestSandbox:
         return new_application
 
 
+    def get_flatpak(self, flatpak_id, **kwargs):
+        """
+        Return flatpak to be used in test.
+
+        :type flatpak_id: str
+        :param flatpak_id: Unique name of flatpak, mandatory format: org.flathub.app
+
+        :return: Flatpak class instance
+        :rtype: <qecore.flatpak.Flatpak>
+        """
+
+        if self.logging:
+            log.info(f"get_flatpak(self, flatpak_id={flatpak_id}")
+
+        flatpak = Flatpak(flatpak_id=flatpak_id, **kwargs)
+        flatpak.shell = self.shell
+        self.applications.append(flatpak)
+        self.default_application = self.default_application or flatpak
+        return flatpak
+
+
     def wait_until_shell_becomes_responsive(self):
         """
         Give some time if shell is not yet loaded fully.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`sandbox.TestSandbox.__init__`.
         """
 
         if self.logging:
@@ -250,7 +326,11 @@ class TestSandbox:
 
     def retrieve_session_data(self):
         """
-        Get session type/desktop data.
+        Get session/system data.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`__init__`.
         """
 
         if self.logging:
@@ -281,22 +361,38 @@ class TestSandbox:
 
 
     def set_up_embedding(self, context):
+        """
+        Set up embeding to the behave html formatter.
+
+        :type context: <behave.runner.Context>
+        :param context: Passed object.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
+
+        """
+
         def embed_data(mime_type, data, caption):
+            # If data is empty we want to finish html tag by at least one character
+            non_empty_data = " " if not data else data
+
             for formatter in context._runner.formatters:
                 if "html" in formatter.name:
-                    self._embed_to = formatter
-
-            if hasattr(self, "_embed_to"):
-                self._embed_to.embedding(mime_type=mime_type, data=data, caption=caption)
-            else:
-                return None
+                    formatter.embedding(mime_type=mime_type, data=non_empty_data, caption=caption)
+                    return
 
         context.embed = embed_data
 
 
     def set_log_start_time(self):
         """
-        Save time. Will be used to retrieve logs from journal.
+        Save time.
+        Will be used to retrieve logs from journal.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
         """
 
         if self.logging:
@@ -307,7 +403,11 @@ class TestSandbox:
 
     def close_yelp(self):
         """
-        Close yelp application that is opened after fresh installation.
+        Close yelp application that is opened after fresh system installation.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
         """
 
         if self.logging:
@@ -320,7 +420,11 @@ class TestSandbox:
 
     def close_initial_setup(self):
         """
-        Close initial setup window that is opened after first login.
+        Close initial setup window that is opened after the first login to the system.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
         """
 
         if self.logging:
@@ -332,6 +436,12 @@ class TestSandbox:
     def set_blank_screen_to_never(self):
         """
         Set blank screen to never. For longer tests it is undesirable for screen to lock.
+
+        .. note::
+
+            This method is called by :func:`before_scenario`.
+            There was never need to have other options,
+            we do not want the system to sleep during the test.
         """
 
         if self.logging:
@@ -342,10 +452,14 @@ class TestSandbox:
 
     def set_max_video_length_to(self, number=600):
         """
-        Set maximum allowed video lenght.
+        Set maximum allowed video length. With default value for 10 minutes.
 
-        @param number : int
-            Maximum video lenght.
+        :type number: int
+        :param number: Maximum video length.
+
+        .. note::
+
+            This method is called by :func:`before_scenario`. You can overwrite the setting.
         """
 
         if self.logging:
@@ -361,6 +475,12 @@ class TestSandbox:
     def display_clock_seconds(self):
         """
         Display clock seconds for better tracking test in video.
+
+        .. note::
+            
+            This method is called by :func:`before_scenario`.
+            There was never need to have other options,
+            as we want to see the seconds ticking during the test.
         """
 
         if self.logging:
@@ -372,6 +492,10 @@ class TestSandbox:
     def return_to_home_workspace(self):
         """
         Return to home workspace.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
         """
 
         if self.logging:
@@ -387,8 +511,13 @@ class TestSandbox:
         """
         Set typing delay so slower machines will not lose characters on type.
 
-        @param number : int
-            Time in between accepted key strokes.
+        :type number: int
+        :param number: Time in between accepted key strokes.
+
+        .. note::
+
+            This method is called by :func:`before_scenario`. You can overwrite the setting.
+
         """
 
         if self.logging:
@@ -401,8 +530,12 @@ class TestSandbox:
         """
         Set debugging to stdout.
 
-        @param true_or_false : bool
-            Decision if debug to stdout or not.
+        :type true_or_false: bool
+        :param true_or_false: Decision if debug to stdout or not.
+
+        .. note::
+
+            This method is called by :func:`before_scenario`. You can overwrite the setting.
         """
 
         if self.logging:
@@ -413,7 +546,11 @@ class TestSandbox:
 
     def copy_data_folder(self):
         """
-        Copy /data folder to /tmp/.
+        Copy data/ directory content to the /tmp/ directory.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
         """
 
         if self.logging:
@@ -428,8 +565,9 @@ class TestSandbox:
         """
         Detect if keyring was setup. If not, setup the keyring with empty password.
 
-        @param context : <context_object>
-            Passed object.
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
         """
 
         if self.logging:
@@ -453,12 +591,16 @@ class TestSandbox:
 
             run(f"touch /tmp/keyring_set")
 
-        run("killall qecore_start_keying")
+        run("killall qecore_start_keyring")
 
 
     def capture_image(self):
         """
         Capture screenshot after failed step.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`after_scenario`.
         """
 
         if self.logging:
@@ -472,10 +614,14 @@ class TestSandbox:
 
     def attach_screenshot_to_report(self, context):
         """
-        Attach screenshot to report upon failed test.
+        Attach screenshot to the html report upon failed test.
 
-        @param context : <context_object>
-            Passed object.
+        :type context: <behave.runner.Context>
+        :param context: Passed object.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`after_scenario`.
         """
 
         if self.logging:
@@ -498,14 +644,20 @@ class TestSandbox:
 
     def attach_image_to_report(self, context, image=None, caption="DefaultCaption"):
         """
-        Attach image to report upon user request.
+        Attach image to the html report upon user request.
 
-        @param context : <context_object>
-            Passed object.
-        @param image : str
-            Location of image/png file.
-        @param caption : str
-            Caption that is to be displayed in test report.
+        :type context: <behave.runner.Context>
+        :param context: Passed object.
+
+        :type image: str
+        :param image: Location of the image/png file.
+
+        :type caption: str
+        :param caption: Caption that is to be displayed in test html report.
+
+        .. note::
+
+            Use this to attach any image to report at any time.
         """
 
         if self.logging:
@@ -524,10 +676,14 @@ class TestSandbox:
 
     def attach_video_to_report(self, context):
         """
-        Attach video to report upon failed test.
+        Attach video to the html report upon failed test.
 
-        @param context : <context_object>
-            Passed object.
+        :type context: <behave.runner.Context>
+        :param context: Passed object.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`after_scenario`.
         """
 
         if self.logging:
@@ -562,10 +718,14 @@ class TestSandbox:
 
     def attach_journal_to_report(self, context):
         """
-        Attach journal to report upon failed test.
+        Attach journal to the html report upon failed test.
 
-        @param context : <context_object>
-            Passed object.
+        :type context: <behave.runner.Context>
+        :param context: Passed object.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`after_scenario`.
         """
 
         if self.logging:
@@ -596,10 +756,14 @@ class TestSandbox:
 
     def attach_abrt_link_to_report(self, context):
         """
-        Attach abrt link to report upon detected abrt FAF report.
+        Attach abrt link to the html report upon detected abrt FAF report.
 
-        @param context : <context_object>
-            Passed object.
+        :type context: <behave.runner.Context>
+        :param context: Passed object.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`after_scenario`.
         """
 
         if self.logging:
@@ -636,11 +800,29 @@ class TestSandbox:
     def wait_for_video_encoding(self, file_name):
         """
         Wait until the video is fully encoded.
-        This is verified by videos changing size.
+        This is verified by video's changing size.
         Once the file is encoded the size will not change anymore.
 
-        @param file_name : str
-            Video location for size verification.
+        :type file_name: str
+        :param file_name: Video location for size verification.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`attach_video_to_report`.
+
+        This fixes some issues with the video and most of the time the video will
+        get passed with all data, in the testing this took between 2-5 seconds.
+        But there still are situations when the encoding is not made in the trivial amount of time
+        mostly on slower machines. Currently the hard cutoff is 60 seconds after that the wait will
+        terminate and the video will get passed as is to the html report.
+
+        This time loss is no issue with few failing tests and has huge advantage of
+        having an entire video with all controling elements (sometimes the video cannot be moved
+        to the middle, or does not have data abouts its length). With many failing tests this might
+        add significant time to the testing time. To prevent waiting for the encoded video
+        and therefore not waiting at all use::
+
+            <qecore.sandbox.TestSandbox>.wait_for_stable_video = False
         """
 
         if self.logging:
@@ -668,12 +850,24 @@ class TestSandbox:
 
     def set_background(self, color=None, background_image=None, background_image_revert=False):
         """
-        Change background to single color. Currently support for white/black/image.
+        Change background to a single color or an image.
 
-        @param color : str
-            String black/white to set as background color.
-        @param background_image : str
-            Image location to be set as background.
+        :type color: str
+        :param color: String black/white to set as background color.
+
+        :type background_image: str
+        :param background_image: Image location to be set as background.
+
+        :type background_image_revert: bool
+        :param background_image_revert: Upon settings this attribute to True,
+            the :func:`after_scenario` will return the background to the original state,
+            after the test.
+
+        To get the wanted color you can pass strings as follows::
+
+            color="black"
+            color="white"
+            color="#FFFFFF" # or any other color represented by hexadecimal
         """
 
         if self.logging:
@@ -702,6 +896,11 @@ class TestSandbox:
             run("gsettings set org.gnome.desktop.background primary-color \"#000000\"")
             run("gsettings set org.gnome.desktop.background secondary-color \"#000000\"")
             run("gsettings set org.gnome.desktop.background color-shading-type \"solid\"")
+        elif "#" in color:
+            run("gsettings set org.gnome.desktop.background picture-uri file://")
+            run(f"gsettings set org.gnome.desktop.background primary-color '{color}'")
+            run(f"gsettings set org.gnome.desktop.background secondary-color '{color}'")
+            run("gsettings set org.gnome.desktop.background color-shading-type \"solid\"")
         else:
             log.info(" ".join((
                 f"Color '{color}' is not defined.",
@@ -714,8 +913,10 @@ class TestSandbox:
         """
         Hide or show application overview.
 
-        @param action : str
-            Hide or show application overview.
+        :type action: str
+        :param action: Hide or show application overview.
+
+        This function takes only 'hide' and 'show' value.
         """
 
         if self.logging:
@@ -742,3 +943,28 @@ class TestSandbox:
             )))
         else:
             assert False, "Unknown option"
+
+
+    # Preserving older api as we have tests with those function calls
+    def preserve_old_api(self):
+        """
+        Preserving old api as the new names of functions and attributes are snake_case complient.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`__init__`.
+        """
+
+        if self.logging:
+            log.info(f"preserve_old_api(self)")
+
+        self.recordVideo = self.record_video
+        self.recordVideoPid = self.record_video_pid
+        self.attachVideo = self.attach_video
+        self.attachVideoOnPass = self.attach_video_on_pass
+        self.attachJournal = self.attach_journal
+        self.attachJournalOnPass = self.attach_journal_on_pass
+        self.attachScreenshot = self.attach_screenshot
+        self.failedTest = self.failed_test
+        self.attachFAF = self.attach_faf
+        self.attachFAFOnPass = self.attach_faf_on_pass
